@@ -1,12 +1,13 @@
-/* $OpenLDAP: pkg/ldap/include/lutil.h,v 1.12.2.4 2000/07/04 17:58:49 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/include/lutil.h,v 1.12.2.7 2001/06/14 00:05:15 kurt Exp $ */
 /*
- * Copyright 1998-2000 The OpenLDAP Foundation, Redwood City, California, USA
+ * Copyright 1998-2001 The OpenLDAP Foundation, Redwood City, California, USA
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms are permitted only
- * as authorized by the OpenLDAP Public License.  A copy of this
- * license is available at http://www.OpenLDAP.org/license.html or
- * in file LICENSE in the top-level directory of the distribution.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.  A copy of this license is available at
+ * http://www.OpenLDAP.org/license.html or in file LICENSE in the
+ * top-level directory of the distribution.
  */
 
 #ifndef _LUTIL_H
@@ -52,7 +53,7 @@ lutil_detach LDAP_P((
 /* entropy.c */
 LDAP_LUTIL_F( int )
 lutil_entropy LDAP_P((
-	char *buf,
+	unsigned char *buf,
 	ber_len_t nbytes ));
 
 /* passwd.c */
@@ -94,6 +95,10 @@ LDAP_LUTIL_F( int )
 lutil_passwd_scheme LDAP_P((
 	const char *scheme ));
 
+LDAP_LUTIL_F( int )
+lutil_salt_format LDAP_P((
+	const char *format ));
+
 /* utils.c */
 LDAP_LUTIL_F( char* )
 lutil_progname LDAP_P((
