@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/controls.c,v 1.19.2.5 2000/09/15 16:13:51 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/controls.c,v 1.19.2.9 2001/07/21 19:01:39 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -336,8 +336,9 @@ ldap_control_dup( const LDAPControl *c )
 		new->ldctl_oid = NULL;
 	}
 
-	if( c->ldctl_value.bv_len > 0 ) {
-		new->ldctl_value.bv_val = (char *) LDAP_MALLOC( c->ldctl_value.bv_len );
+	if( c->ldctl_value.bv_val != NULL ) {
+		new->ldctl_value.bv_val =
+			(char *) LDAP_MALLOC( c->ldctl_value.bv_len + 1 );
 
 		if(new->ldctl_value.bv_val == NULL) {
 			if(new->ldctl_oid != NULL) {
@@ -347,10 +348,12 @@ ldap_control_dup( const LDAPControl *c )
 			return NULL;
 		}
 		
+		new->ldctl_value.bv_len = c->ldctl_value.bv_len;
+
 		AC_MEMCPY( new->ldctl_value.bv_val, c->ldctl_value.bv_val, 
 			c->ldctl_value.bv_len );
 
-		new->ldctl_value.bv_len = c->ldctl_value.bv_len;
+		new->ldctl_value.bv_val[new->ldctl_value.bv_len] = '\0';
 
 	} else {
 		new->ldctl_value.bv_len = 0;
@@ -410,7 +413,7 @@ ldap_create_control(
 	LDAPControl *ctrl;
 	struct berval *bvalp;
 
-	if ( requestOID == NULL || ber == NULL || ctrlp == NULL ) {
+	if ( requestOID == NULL || ctrlp == NULL ) {
 		return LDAP_PARAM_ERROR;
 	}
 
@@ -438,3 +441,34 @@ ldap_create_control(
 	*ctrlp = ctrl;
 	return LDAP_SUCCESS;
 }
+
+/*
+ * check for critical client controls and bitch if present
+ * if we ever support critical controls, we'll have to
+ * find a means for maintaining per API call control
+ * information.
+ */
+int ldap_int_client_controls( LDAP *ld, LDAPControl **ctrls )
+{
+	LDAPControl *const *c;
+
+	assert( ld != NULL );
+
+	if( ctrls == NULL ) {
+		/* use default server controls */
+		ctrls = ld->ld_cctrls;
+	}
+
+	if( ctrls == NULL || *ctrls == NULL ) {
+		return LDAP_SUCCESS;
+	}
+
+	for( c = ctrls ; *c != NULL; c++ ) {
+		if( (*c)->ldctl_iscritical ) {
+			ld->ld_errno = LDAP_NOT_SUPPORTED;
+			return ld->ld_errno;
+		}
+	}
+
+	return LDAP_SUCCESS;
+}
