@@ -1,5 +1,5 @@
 /* ldbm.c - ldap dbm compatibility routines */
-/* $OpenLDAP: pkg/ldap/libraries/libldbm/ldbm.c,v 1.30.8.8 2000/09/20 19:32:41 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldbm/ldbm.c,v 1.30.8.13 2001/08/28 17:18:34 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -50,8 +50,10 @@ ldbm_datum_dup( LDBM ldbm, Datum data )
 		return( dup );
 	}
 	dup.dsize = data.dsize;
-	if ( (dup.dptr = (char *) malloc( data.dsize )) != NULL )
+
+	if ( (dup.dptr = (char *) malloc( data.dsize )) != NULL ) {
 		AC_MEMCPY( dup.dptr, data.dptr, data.dsize );
+	}
 
 	return( dup );
 }
@@ -59,17 +61,18 @@ ldbm_datum_dup( LDBM ldbm, Datum data )
 static int ldbm_initialized = 0;
 
 #ifdef HAVE_BERKELEY_DB_THREAD
-#define LDBM_LOCK	((void)0)
-#define LDBM_UNLOCK	((void)0)
+#define LDBM_LOCK	((void) 0)
+#define LDBM_UNLOCK	((void) 0)
 #else
-
-/* Only DB2 or DB3 with DB_THREAD is thread-free */
 static ldap_pvt_thread_mutex_t ldbm_big_mutex;
 #define LDBM_LOCK	(ldap_pvt_thread_mutex_lock(&ldbm_big_mutex))
 #define LDBM_UNLOCK	(ldap_pvt_thread_mutex_unlock(&ldbm_big_mutex))
-
 #endif
 
+#if !defined( HAVE_BERKELEY_DB ) || (DB_VERSION_MAJOR < 3)
+				/*  a dbEnv for BERKELEYv2  */
+DB_ENV *ldbm_Env = NULL;	/* real or fake, depending on db and version */
+#endif
 
 
 /*******************************************************************
@@ -83,7 +86,8 @@ static ldap_pvt_thread_mutex_t ldbm_big_mutex;
 void *
 ldbm_malloc( size_t size )
 {
-	return( calloc( 1, size ));
+	/* likely should use ber_mem* routines */
+	return( calloc( 1, size ) );
 }
 
 #ifdef LDAP_SYSLOG
@@ -98,87 +102,79 @@ ldbm_db_errcall( const char *prefix, char *message )
 #endif
 }
 
-/*  a dbEnv for BERKELEYv2  */
-DB_ENV           *ldbm_Env = NULL;
+#if DB_VERSION_MAJOR < 3
 
 int ldbm_initialize( const char* home )
 {
-	int     err;
+	int	err;
 	u_int32_t	envFlags;
 
 	if(ldbm_initialized++) return 1;
 
-#if DB_VERSION_MAJOR < 3
+	{
+		char *version;
+		int major, minor, patch;
+		version = db_version( &major, &minor, &patch );
+
+		if( major != DB_VERSION_MAJOR ||
+			minor < DB_VERSION_MINOR )
+		{
+#ifdef LDAP_SYSLOG
+			char error[BUFSIZ];
+
+			sprintf( error, "%s (%d)\n", STRERROR( err ), err );
+
+			syslog( LOG_INFO,
+				"ldbm_initialize(): version mismatch\nexpected: %s\ngot: %s\n",
+				DB_VERSION_STRING,
+				version );
+#endif
+
+			return 1;
+		}
+	}
+
+#ifndef HAVE_BERKELEY_DB_THREAD
+	ldap_pvt_thread_mutex_init( &ldbm_big_mutex );
+#endif
+
 	ldbm_Env = calloc( 1, sizeof( DB_ENV ));
 
 	if( ldbm_Env == NULL ) return 1;
 
-	ldbm_Env->db_errcall   = ldbm_db_errcall;
-	ldbm_Env->db_errpfx    = "==>";
-#else
-	ldbm_Env = NULL;
+	ldbm_Env->db_errcall	= ldbm_db_errcall;
+	ldbm_Env->db_errpfx		= "==>";
+
+	envFlags = DB_CREATE;
+
+	/* add optional flags */
+#ifdef DB_PRIVATE
+	envFlags |= DB_PRIVATE;
 #endif
 
-	envFlags = 
-#if defined( DB_PRIVATE )
-		DB_PRIVATE |
+#ifdef HAVE_BERKELEY_DB_THREAD
+	envFlags |= DB_THREAD | DB_INIT_CDB | DB_INIT_MPOOL;
+#ifdef DB_MPOOL_PRIVATE
+	envFlags |= DB_MPOOL_PRIVATE;
 #endif
-#if defined( HAVE_BERKELEY_DB_THREAD )
-		DB_THREAD |
 #endif
-		DB_CREATE;
 
-#if DB_VERSION_MAJOR >= 3
-	err = db_env_create( &ldbm_Env, 0 );
-#else
 	envFlags |= DB_USE_ENVIRON;
 	err = db_appinit( home, NULL, ldbm_Env, envFlags );
-#endif
 
 	if ( err ) {
+#ifdef LDAP_SYSLOG
 		char error[BUFSIZ];
 
 		sprintf( error, "%s (%d)\n", STRERROR( err ), err );
 
-#ifdef LDAP_SYSLOG
 		syslog( LOG_INFO,
-#if DB_VERSION_MAJOR >= 3
-			"ldbm_initialize(): FATAL error in db_env_create() : %s\n",
-#else
 			"ldbm_initialize(): FATAL error in db_appinit() : %s\n",
-#endif
 			error );
 #endif
 	 	return( 1 );
 	}
 
-#if DB_VERSION_MAJOR > 2
-	ldbm_Env->set_errcall( ldbm_Env, ldbm_db_errcall );
-	ldbm_Env->set_errpfx( ldbm_Env, "==>" );
-
-	envFlags |= DB_INIT_MPOOL | DB_USE_ENVIRON;
-
-#if (DB_VERSION_MAJOR > 3) || (DB_VERSION_MINOR >= 1)
-        err = ldbm_Env->open( ldbm_Env, home, envFlags, 0 );
-#else
-        err = ldbm_Env->open( ldbm_Env, home, NULL, envFlags, 0 );
-#endif
-        if ( err != 0 )
-        {
-            char error[BUFSIZ];
-
-            sprintf( error, "%s (%d)\n", STRERROR( err ), err );
-
-#ifdef LDAP_SYSLOG
-            syslog( LOG_INFO,
-                    "ldbm_initialize(): FATAL error in dbEnv->open() : %s\n",
-                    error );
-#endif
-		    ldbm_Env->close( ldbm_Env, 0 );
-            return( 1 );
-	}
-#endif
-
 	return 0;
 }
 
@@ -186,15 +182,35 @@ int ldbm_shutdown( void )
 {
 	if( !ldbm_initialized ) return 1;
 
-#if DB_VERSION_MAJOR >= 3
-	ldbm_Env->close( ldbm_Env, 0 );
-#else
 	db_appexit( ldbm_Env );
+
+#ifndef HAVE_BERKELEY_DB_THREAD
+	ldap_pvt_thread_mutex_destroy( &ldbm_big_mutex );
 #endif
 
 	return 0;
 }
 
+#else  /* Berkeley v3 or greater */
+
+
+int ldbm_initialize( const char * home )
+{
+	/* v3 uses ldbm_initialize_env */
+	return 0;
+}
+
+
+int ldbm_shutdown( void )
+{
+	return 0;
+}
+
+
+#endif
+
+
+
 #else  /* some DB other than Berkeley V2 or greater */
 
 int ldbm_initialize( const char * home )
@@ -218,6 +234,86 @@ int ldbm_shutdown( void )
 #endif /* HAVE_BERKELEY_DB */
 
 
+#if defined( HAVE_BERKELEY_DB ) && (DB_VERSION_MAJOR >= 3)
+
+
+DB_ENV *ldbm_initialize_env(const char *home, int dbcachesize, int *envdirok)
+{
+	DB_ENV *env = NULL;    
+	int     err;
+	u_int32_t	envFlags;
+
+	envFlags = 
+#if defined( DB_PRIVATE )	/* comment out DB_PRIVATE setting to use */
+	DB_PRIVATE |		/* db_stat to view cache behavior */
+#endif
+#if defined( HAVE_BERKELEY_DB_THREAD )
+	DB_THREAD |
+#endif
+	DB_CREATE;
+	
+	err = db_env_create( &env, 0 );
+
+	if ( err ) {
+		char error[BUFSIZ];
+
+		sprintf( error, "%s (%d)\n", STRERROR( err ), err );
+
+#ifdef LDAP_SYSLOG
+		syslog( LOG_INFO, "ldbm_initialize_env(): FATAL error in db_env_create() : %s\n", error );
+#endif
+		return( NULL );
+	}
+
+	env->set_errcall( env, ldbm_db_errcall );
+	env->set_errpfx( env, "==>" );
+	if (dbcachesize)
+		env->set_cachesize( env, 0, dbcachesize, 0 );
+
+	envFlags |= DB_INIT_MPOOL | DB_INIT_CDB | DB_USE_ENVIRON;
+
+	err = env->open( env, home, envFlags, 0 );
+
+	if ( err != 0 )
+	{
+		char error[BUFSIZ];
+
+		sprintf( error, "%s (%d)\n", STRERROR( err ), err );
+
+#ifdef LDAP_SYSLOG
+		syslog(	LOG_INFO,
+			"ldbm_initialize_env(): FATAL error in dbEnv->open() : %s\n",
+			error );
+#endif
+		env->close( env, 0 );
+		return( NULL );
+	}
+
+	*envdirok = 1;
+	return env;
+}
+
+void ldbm_shutdown_env(DB_ENV *env)
+{
+	env->close( env, 0 );
+}
+
+
+#else
+
+DB_ENV *ldbm_initialize_env(const char *home, int dbcachesize, int *envdirok)
+{
+	return ldbm_Env;
+}
+
+void ldbm_shutdown_env(DB_ENV *env)
+{
+}
+
+
+#endif
+
+
 #if defined( LDBM_USE_DBHASH ) || defined( LDBM_USE_DBBTREE )
 
 /*****************************************************************
@@ -227,38 +323,46 @@ int ldbm_shutdown( void )
  *****************************************************************/
 
 LDBM
-ldbm_open( char *name, int rw, int mode, int dbcachesize )
+ldbm_open( DB_ENV *env, char *name, int rw, int mode, int dbcachesize )
 {
 	LDBM		ret = NULL;
 
 #if DB_VERSION_MAJOR >= 3
-       int err;
-       LDBM_LOCK;
-       err = db_create( &ret, ldbm_Env, 0 );
-       if ( err != 0 )
-       {
-           char error[BUFSIZ];
+	int err;
 
-           sprintf( error, "%s (%d)\n", STRERROR( err ), err );
+	LDBM_LOCK;
 
-           (void)ret->close(ret, 0);
-           return NULL;
-       }
+	err = db_create( &ret, env, 0 );
+	if ( err != 0 ) {
+		(void)ret->close(ret, 0);
+		LDBM_UNLOCK;
 
-       ret->set_pagesize( ret, DEFAULT_DB_PAGE_SIZE );
-       ret->set_malloc( ret, ldbm_malloc );
-       ret->set_cachesize( ret, 0, dbcachesize, 0 );
-       err = ret->open( ret, name, NULL, DB_TYPE, rw, mode);
-       LDBM_UNLOCK;
-       if ( err != 0 )
-       {
-           char error[BUFSIZ];
+		return NULL;
+	}
 
-           sprintf( error, "%s (%d)\n", STRERROR( err ), err );
+	ret->set_pagesize( ret, DEFAULT_DB_PAGE_SIZE );
 
-           (void)ret->close(ret, 0);
-           return NULL;
-       }
+	/* likely should use ber_mem* routines */
+#if DB_VERSION_MINOR >= 3
+	ret->set_alloc( ret, ldbm_malloc, NULL, NULL );
+#else
+	ret->set_malloc( ret, ldbm_malloc );
+#endif
+
+	/* ret->set_cachesize( ret, 0, dbcachesize, 0 ); */
+
+	err = ret->open( ret, name, NULL, DB_TYPE, rw, mode);
+
+	if ( err != 0 ) {
+		int tmp = errno;
+		(void)ret->close(ret, 0);
+		errno = tmp;
+
+		LDBM_UNLOCK;
+		return NULL;
+	}
+
+	LDBM_UNLOCK;
  
 #elif DB_VERSION_MAJOR >= 2
 	DB_INFO dbinfo;
@@ -272,15 +376,16 @@ ldbm_open( char *name, int rw, int mode, int dbcachesize )
 	 */
 #else
 	/* set db_cachesize of MPOOL is NOT being used. */
-	if (( ldbm_Env == NULL ) || ( ldbm_Env->mp_info == NULL ))
+	if (( ldbm_Env == NULL ) || ( ldbm_Env->mp_info == NULL )) {
 		dbinfo.db_cachesize = dbcachesize;
+	}
 #endif
 
-	dbinfo.db_pagesize  = DEFAULT_DB_PAGE_SIZE;
-	dbinfo.db_malloc    = ldbm_malloc;
+	dbinfo.db_pagesize	= DEFAULT_DB_PAGE_SIZE;
+	dbinfo.db_malloc	= ldbm_malloc;
 
 	LDBM_LOCK;
-    (void) db_open( name, DB_TYPE, rw, mode, ldbm_Env, &dbinfo, &ret );
+	(void) db_open( name, DB_TYPE, rw, mode, ldbm_Env, &dbinfo, &ret );
 	LDBM_UNLOCK;
 
 #else
@@ -303,10 +408,9 @@ ldbm_open( char *name, int rw, int mode, int dbcachesize )
 	LDBM_LOCK;
 	ret = dbopen( name, rw, mode, DB_TYPE, info );
 	LDBM_UNLOCK;
-
 #endif
 
-	return( ret );
+	return ret;
 }
 
 void
@@ -314,7 +418,7 @@ ldbm_close( LDBM ldbm )
 {
 	LDBM_LOCK;
 #if DB_VERSION_MAJOR >= 3
-        ldbm->close( ldbm, 0 );
+	ldbm->close( ldbm, 0 );
 #elif DB_VERSION_MAJOR >= 2
 	(*ldbm->close)( ldbm, 0 );
 #else
@@ -338,13 +442,17 @@ ldbm_fetch( LDBM ldbm, Datum key )
 	int	rc;
 
 	LDBM_LOCK;
+
 #if DB_VERSION_MAJOR >= 3
-        ldbm_datum_init( data );
+	ldbm_datum_init( data );
 
-        data.flags = DB_DBT_MALLOC;
+	data.flags = DB_DBT_MALLOC;
 
-        if ( (rc = ldbm->get( ldbm, NULL, &key, &data, 0 )) != 0 ) {
-            ldbm_datum_free( ldbm, data );
+	if ( (rc = ldbm->get( ldbm, NULL, &key, &data, 0 )) != 0 ) {
+		ldbm_datum_free( ldbm, data );
+		data.dptr = NULL;
+		data.dsize = 0;
+	}
 
 #elif DB_VERSION_MAJOR >= 2
 	ldbm_datum_init( data );
@@ -353,16 +461,20 @@ ldbm_fetch( LDBM ldbm, Datum key )
 
 	if ( (rc = (*ldbm->get)( ldbm, NULL, &key, &data, 0 )) != 0 ) {
 		ldbm_datum_free( ldbm, data );
+		data.dptr = NULL;
+		data.dsize = 0;
+	}
 #else
+
 	if ( (rc = (*ldbm->get)( ldbm, &key, &data, 0 )) == 0 ) {
 		/* Berkeley DB 1.85 don't malloc the data for us */
 		/* duplicate it for to ensure reentrancy */
 		data = ldbm_datum_dup( ldbm, data );
 	} else {
-#endif
 		data.dptr = NULL;
 		data.dsize = 0;
 	}
+#endif
 
 	LDBM_UNLOCK;
 
@@ -377,18 +489,12 @@ ldbm_store( LDBM ldbm, Datum key, Datum data, int flags )
 	LDBM_LOCK;
 
 #if DB_VERSION_MAJOR >= 3
-        rc = ldbm->put( ldbm, NULL, &key, &data, flags & ~LDBM_SYNC );
-       if ( rc != 0 )
-       {
-           char error[BUFSIZ];
-
-           sprintf( error, "%s (%d)\n", STRERROR( rc ), rc );
-       }
-       rc = (-1) * rc;
+	rc = ldbm->put( ldbm, NULL, &key, &data, flags & ~LDBM_SYNC );
+	rc = (-1) * rc;
 
 #elif DB_VERSION_MAJOR >= 2
 	rc = (*ldbm->put)( ldbm, NULL, &key, &data, flags & ~LDBM_SYNC );
-	rc = (-1 ) * rc;
+	rc = (-1) * rc;
 #else
 	rc = (*ldbm->put)( ldbm, &key, &data, flags & ~LDBM_SYNC );
 #endif
@@ -410,10 +516,10 @@ ldbm_delete( LDBM ldbm, Datum key )
 
 #if DB_VERSION_MAJOR >= 3
 	rc = ldbm->del( ldbm, NULL, &key, 0 );
-	rc = (-1 ) * rc;
+	rc = (-1) * rc;
 #elif DB_VERSION_MAJOR >= 2
 	rc = (*ldbm->del)( ldbm, NULL, &key, 0 );
-	rc = (-1 ) * rc;
+	rc = (-1) * rc;
 #else
 	rc = (*ldbm->del)( ldbm, &key, 0 );
 #endif
@@ -441,9 +547,9 @@ ldbm_firstkey( LDBM ldbm, LDBMCursor **dbch )
 
 	/* acquire a cursor for the DB */
 # if DB_VERSION_MAJOR >= 3
-        if ( ldbm->cursor( ldbm, NULL, &dbci, 0 ) )
+	if ( ldbm->cursor( ldbm, NULL, &dbci, 0 ) )
 # elif defined( DB_VERSION_MAJOR ) && defined( DB_VERSION_MINOR ) && \
-    (DB_VERSION_MAJOR == 2 && DB_VERSION_MINOR < 6)
+	(DB_VERSION_MAJOR == 2 && DB_VERSION_MINOR < 6)
 
 	if ( (*ldbm->cursor)( ldbm, NULL, &dbci ))
 
@@ -543,7 +649,7 @@ ldbm_errno( LDBM ldbm )
  *****************************************************************/
 
 LDBM
-ldbm_open( char *name, int rw, int mode, int dbcachesize )
+ldbm_open( DB_ENV *env, char *name, int rw, int mode, int dbcachesize )
 {
 	LDBM		db;
 #ifdef HAVE_ST_BLKSIZE
@@ -717,7 +823,7 @@ ldbm_errno( LDBM ldbm )
  *****************************************************************/
 
 LDBM
-ldbm_open( char *name, int rw, int mode, int dbcachesize )
+ldbm_open( DB_ENV *env, char *name, int rw, int mode, int dbcachesize )
 {
 	LDBM		db;
 
@@ -790,15 +896,13 @@ ldbm_close( LDBM ldbm )
 void
 ldbm_sync( LDBM ldbm )
 {
-
 	/* XXX: Not sure if this is re-entrant need to check code, if so
 	 * you can leave LOCKS out.
 	 */
 
 	LDBM_LOCK;
 	mdbm_sync( ldbm );
-        LDBM_UNLOCK;
-
+	LDBM_UNLOCK;
 }
 
 
@@ -837,9 +941,7 @@ ldbm_fetch( LDBM ldbm, Datum key )
 		if ( d.dsize > 0 ) {
 
 			if ( k.val.dptr != NULL ) {
-			    
-			    free( k.val.dptr );
-
+				free( k.val.dptr );
 			}
 
 			if ( (k.val.dptr = malloc( d.dsize )) != NULL ) {
@@ -942,7 +1044,6 @@ ldbm_delete( LDBM ldbm, Datum key )
 #endif
 
 	return( rc );
-
 }
 
 
@@ -951,7 +1052,6 @@ ldbm_delete( LDBM ldbm, Datum key )
 static Datum
 ldbm_get_next( LDBM ldbm, kvpair (*fptr)(MDBM *, kvpair) ) 
 {
-
 	kvpair	out;
 	kvpair	in;
 	Datum	ret;
@@ -976,55 +1076,39 @@ ldbm_get_next( LDBM ldbm, kvpair (*fptr)(MDBM *, kvpair) )
 	out = fptr( ldbm, in );
 
 	if (out.key.dsize > 0) {
+		ret.dsize = out.key.dsize - delta;
 
-	    ret.dsize = out.key.dsize - delta;
-	    if ((ret.dptr = (char *)malloc(ret.dsize)) == NULL) { 
-
-		ret.dsize = 0;
-		ret.dptr = NULL;
-
-	    } else {
-
-		AC_MEMCPY(ret.dptr, (void *)(out.key.dptr + delta),
-		       ret.dsize );
+		if ((ret.dptr = (char *)malloc(ret.dsize)) == NULL) { 
+			ret.dsize = 0;
+			ret.dptr = NULL;
 
+		} else {
+			AC_MEMCPY(ret.dptr, (void *)(out.key.dptr + delta),
+				ret.dsize );
 	    }
-
 	}
 
 	/* LDBM_UNLOCK; */
 	
 	free(in.key.dptr);
-
 	return ret;
-
 }
 
-
-
-
 Datum
 ldbm_firstkey( LDBM ldbm, LDBMCursor **dbcp )
 {
-
 	return ldbm_get_next( ldbm, mdbm_first );
-
 }
 
-
-
-
 Datum
 ldbm_nextkey( LDBM ldbm, Datum key, LDBMCursor *dbcp )
 {
-
 	/* XXX:
-	 * don't know if this will affect the LDAP server opertaion 
+	 * don't know if this will affect the LDAP server operation 
 	 * but mdbm cannot take and input key.
 	 */
 
 	return ldbm_get_next( ldbm, mdbm_next );
-
 }
 
 int
@@ -1032,7 +1116,6 @@ ldbm_errno( LDBM ldbm )
 {
 	/* XXX: best we can do with current  mdbm interface */
 	return( errno );
-
 }
 
 
@@ -1048,7 +1131,7 @@ ldbm_errno( LDBM ldbm )
 
 /* ARGSUSED */
 LDBM
-ldbm_open( char *name, int rw, int mode, int dbcachesize )
+ldbm_open( DB_ENV *env, char *name, int rw, int mode, int dbcachesize )
 {
 	LDBM ldbm;
 
