@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/compare.c,v 1.17.2.6 2000/09/08 15:01:33 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/compare.c,v 1.17.2.8 2001/07/21 19:01:41 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -37,6 +37,7 @@ do_compare(
 	Backend	*be;
 	int rc = LDAP_SUCCESS;
 	const char *text = NULL;
+	int manageDSAit;
 
 	ava.aa_desc = NULL;
 	desc.bv_val = NULL;
@@ -93,19 +94,21 @@ do_compare(
 		goto cleanup;
 	}
 
-	if( ndn == '\0' ) {
+	if( *ndn == '\0' ) {
 		Debug( LDAP_DEBUG_ANY, "do_compare: root dse!\n", 0, 0, 0 );
 		send_ldap_result( conn, op, rc = LDAP_UNWILLING_TO_PERFORM,
 			NULL, "compare upon the root DSE not supported", NULL, NULL );
 		goto cleanup;
 	}
 
+	manageDSAit = get_manageDSAit( op );
+
 	/*
 	 * We could be serving multiple database backends.  Select the
 	 * appropriate one, or send a referral to our "referral server"
 	 * if we don't hold it.
 	 */
-	if ( (be = select_backend( ndn )) == NULL ) {
+	if ( (be = select_backend( ndn, manageDSAit )) == NULL ) {
 		send_ldap_result( conn, op, rc = LDAP_REFERRAL,
 			NULL, NULL, default_referral, NULL );
 		rc = 1;
