@@ -1,5 +1,5 @@
 /* config.c - configuration file handling routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/config.c,v 1.47.2.20 2001/05/20 15:40:10 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/config.c,v 1.47.2.23 2001/07/26 23:13:27 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -41,6 +41,7 @@ char		*default_search_base = NULL;
 char		*default_search_nbase = NULL;
 
 ber_len_t sockbuf_max_incoming = SLAP_SB_MAX_INCOMING_DEFAULT;
+ber_len_t sockbuf_max_incoming_auth= SLAP_SB_MAX_INCOMING_AUTH;
 
 char   *slapd_pid_file  = NULL;
 char   *slapd_args_file = NULL;
@@ -172,15 +173,9 @@ read_config( const char *fname )
 		} else if ( strcasecmp( cargv[0], "sockbuf_max_incoming" ) == 0 ) {
 			long max;
 			if ( cargc < 2 ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG(( "config", LDAP_LEVEL_CRIT,
-					   "%s: line %d: missing max in \"sockbuf_max_incoming <bytes\" line\n",
-					   fname, lineno ));
-#else
 				Debug( LDAP_DEBUG_ANY,
 					   "%s: line %d: missing max in \"sockbuf_max_incoming <bytes\" line\n",
 				    fname, lineno, 0 );
-#endif
 
 				return( 1 );
 			}
@@ -188,23 +183,40 @@ read_config( const char *fname )
 			max = atol( cargv[1] );
 
 			if( max < 0 ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG(( "config", LDAP_LEVEL_CRIT,
-					   "%s: line %d: invalid max value (%ld) in "
-					   "\"sockbuf_max_incoming <bytes>\" line.\n",
-					   fname, lineno, max ));
-#else
 				Debug( LDAP_DEBUG_ANY,
 					"%s: line %d: invalid max value (%ld) in "
 					"\"sockbuf_max_incoming <bytes>\" line.\n",
 				    fname, lineno, max );
-#endif
 
 				return( 1 );
 			}
 
 			sockbuf_max_incoming = max;
 
+		/* set sockbuf max authenticated */
+		} else if ( strcasecmp( cargv[0], "sockbuf_max_incoming_auth" ) == 0 ) {
+			long max;
+			if ( cargc < 2 ) {
+				Debug( LDAP_DEBUG_ANY,
+					   "%s: line %d: missing max in \"sockbuf_max_incoming_auth <bytes>\" line\n",
+				    fname, lineno, 0 );
+
+				return( 1 );
+			}
+
+			max = atol( cargv[1] );
+
+			if( max < 0 ) {
+				Debug( LDAP_DEBUG_ANY,
+					"%s: line %d: invalid max value (%ld) in "
+					"\"sockbuf_max_incoming_auth <bytes>\" line.\n",
+				    fname, lineno, max );
+
+				return( 1 );
+			}
+
+			sockbuf_max_incoming_auth = max;
+
 		/* default search base */
 		} else if ( strcasecmp( cargv[0], "defaultSearchBase" ) == 0 ) {
 			if ( cargc < 2 ) {
@@ -308,6 +320,17 @@ read_config( const char *fname )
 				default_passwd_hash = ch_strdup( cargv[1] );
 			}
 
+		} else if ( strcasecmp( cargv[0], "password-crypt-salt-format" ) == 0 ) 
+		{
+			if ( cargc < 2 ) {
+				Debug( LDAP_DEBUG_ANY, "%s: line %d: missing format in "
+					"\"password-crypt-salt-format <format>\" line\n",
+				    fname, lineno, 0 );
+				return 1;
+			}
+
+			lutil_salt_format( cargv[1] );
+
 		/* set SASL host */
 		} else if ( strcasecmp( cargv[0], "sasl-host" ) == 0 ) {
 			if ( cargc < 2 ) {
@@ -908,11 +931,13 @@ read_config( const char *fname )
 "%s: line %d: replica line must appear inside a database definition (ignored)\n",
 				    fname, lineno, 0 );
 			} else {
+				int nr = -1;
+
 				for ( i = 1; i < cargc; i++ ) {
 					if ( strncasecmp( cargv[i], "host=", 5 )
 					    == 0 ) {
-						charray_add( &be->be_replica,
-							     cargv[i] + 5 );
+						nr = add_replica_info( be, 
+							cargv[i] + 5 );
 						break;
 					}
 				}
@@ -920,6 +945,31 @@ read_config( const char *fname )
 					Debug( LDAP_DEBUG_ANY,
 		    "%s: line %d: missing host in \"replica\" line (ignored)\n",
 					    fname, lineno, 0 );
+
+				} else if ( nr == -1 ) {
+					Debug( LDAP_DEBUG_ANY,
+		"%s: line %d: unable to add replica \"%s\" (ignored)\n",
+						fname, lineno, cargv[i] + 5 );
+				} else {
+					for ( i = 1; i < cargc; i++ ) {
+						if ( strncasecmp( cargv[i], "suffix=", 7 ) == 0 ) {
+							char *nsuffix = ch_strdup( cargv[i] + 7 );
+							if ( dn_normalize( nsuffix ) != NULL ) {
+								if ( be_issuffix( be, nsuffix ) ) {
+									charray_add( &be->be_replica[nr]->ri_nsuffix, nsuffix );
+								} else {
+									Debug( LDAP_DEBUG_ANY,
+											"%s: line %d: suffix \"%s\" in \"replica\" line is not valid for backend (ignored)\n",
+											fname, lineno, cargv[i] + 7 );
+								}
+							} else {
+								Debug( LDAP_DEBUG_ANY,
+										 "%s: line %d: unable to normalize suffix in \"replica\" line (ignored)\n",
+										 fname, lineno, 0 );
+							}
+							free( nsuffix );
+						}
+					}
 				}
 			}
 
