@@ -1,5 +1,5 @@
 /* filter.c - routines for parsing and dealing with filters */
-/* $OpenLDAP: pkg/ldap/servers/slapd/filter.c,v 1.13.4.6 2000/10/11 02:43:58 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/filter.c,v 1.13.4.8 2001/06/22 23:59:28 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -265,7 +265,7 @@ get_filter(
 		break;
 
 	default:
-		(void) ber_skip_tag( ber, &len );
+		(void) ber_scanf( ber, "x" ); /* skip the element */
 		Debug( LDAP_DEBUG_ANY, "get_filter: unknown filter type=%lu\n",
 		       f->f_choice, 0, 0 );
 		f->f_choice = SLAPD_FILTER_COMPUTED;
@@ -443,7 +443,10 @@ get_substring_filter(
 		switch ( tag ) {
 		case LDAP_SUBSTRING_INITIAL:
 			Debug( LDAP_DEBUG_FILTER, "  INITIAL\n", 0, 0, 0 );
-			if ( f->f_sub_initial != NULL ) {
+			if ( f->f_sub_initial != NULL
+				|| f->f_sub_any != NULL 
+				|| f->f_sub_final != NULL )
+			{
 				ber_bvfree( value );
 				goto return_error;
 			}
@@ -461,6 +464,12 @@ get_substring_filter(
 
 		case LDAP_SUBSTRING_ANY:
 			Debug( LDAP_DEBUG_FILTER, "  ANY\n", 0, 0, 0 );
+
+			if ( f->f_sub_final != NULL ) {
+				ber_bvfree( value );
+				goto return_error;
+			}
+
 			if( ber_bvecadd( &f->f_sub_any, value ) < 0 ) {
 				ber_bvfree( value );
 				goto return_error;
@@ -478,10 +487,12 @@ get_substring_filter(
 
 		case LDAP_SUBSTRING_FINAL:
 			Debug( LDAP_DEBUG_FILTER, "  FINAL\n", 0, 0, 0 );
+
 			if ( f->f_sub_final != NULL ) {
 				ber_bvfree( value );
 				goto return_error;
 			}
+
 			f->f_sub_final = value;
 
 			if( fstr ) {
