@@ -1,5 +1,5 @@
 /* schema_init.c - init builtin schema */
-/* $OpenLDAP: pkg/ldap/servers/slapd/schema_init.c,v 1.42.2.29 2001/05/09 22:00:53 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/schema_init.c,v 1.42.2.32 2001/07/25 22:53:36 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -408,6 +408,7 @@ bitStringValidate(
 	if( in->bv_len < 3 ) {
 		return LDAP_INVALID_SYNTAX;
 	}
+
 	if( in->bv_val[0] != 'B' ||
 		in->bv_val[1] != '\'' ||
 		in->bv_val[in->bv_len-1] != '\'' )
@@ -474,7 +475,8 @@ UTF8casecmp(
 	struct berval *left )
 {
 	ber_len_t r, l;
-	int rlen, llen;
+	ber_len_t rlen, llen;
+	ber_len_t rslen, lslen;
 	ldap_unicode_t ru, lu;
 	ldap_unicode_t ruu, luu;
 
@@ -1892,8 +1894,6 @@ countryStringValidate(
 	Syntax *syntax,
 	struct berval *val )
 {
-	ber_len_t i;
-
 	if( val->bv_len != 2 ) return LDAP_INVALID_SYNTAX;
 
 	if( !SLAP_PRINTABLE(val->bv_val[0]) ) {
@@ -3182,8 +3182,6 @@ numericStringValidate(
 {
 	ber_len_t i;
 
-	/* disallow empty numeric strings */
-
 	for(i=0; i < in->bv_len; i++) {
 		if( !SLAP_NUMERIC(in->bv_val[i]) ) {
 			return LDAP_INVALID_SYNTAX;
@@ -3218,8 +3216,8 @@ numericStringNormalize(
 		}
 	}
 
-	assert( newval->bv_val < p );
-	assert( q <= p );
+	/* we should have copied no more then is in val */
+	assert( (q - newval->bv_val) <= (p - val->bv_val) );
 
 	/* null terminate */
 	*q = '\0';
