@@ -1,5 +1,5 @@
 /* slap.h - stand alone ldap server include file */
-/* $OpenLDAP: pkg/ldap/servers/slapd/slap.h,v 1.86.2.22 2001/05/10 04:43:48 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/slap.h,v 1.86.2.26 2001/07/26 23:13:28 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -108,7 +108,7 @@ LDAP_BEGIN_DECL
 #define SLAP_PRINTABLE(c)	( ASCII_ALNUM(c) || (c) == '\'' || \
 	(c) == '(' || (c) == ')' || (c) == '+' || (c) == ',' || \
 	(c) == '-' || (c) == '.' || (c) == '/' || (c) == ':' || \
-	(c) == '?' || (c) == ' ' )
+	(c) == '?' || (c) == ' ' || (c) == '=' )
 #define SLAP_PRINTABLES(c)	( SLAP_PRINTABLE(c) || (c) == '$' )
 
 /* must match in schema_init.c */
@@ -216,6 +216,8 @@ typedef struct slap_ssf_set {
 #define SLAP_SCHERR_MR_NOT_FOUND	12
 #define SLAP_SCHERR_SYN_NOT_FOUND	13
 #define SLAP_SCHERR_MR_INCOMPLETE	14
+#define SLAP_SCHERR_NOT_SUPPORTED	15
+#define SLAP_SCHERR_BAD_DESCR	16
 
 typedef struct slap_oid_macro {
 	struct berval som_oid;
@@ -382,6 +384,7 @@ typedef struct slap_attribute_type {
 #define is_at_operational(at)	((at)->sat_usage)
 #define is_at_single_value(at)	((at)->sat_single_value)
 #define is_at_collective(at)	((at)->sat_collective)
+#define is_at_obsolete(at)		((at)->sat_obsolete)
 #define is_at_no_user_mod(at)	((at)->sat_no_user_mod)
 
 typedef struct slap_object_class {
@@ -804,6 +807,11 @@ LDAP_SLAPD_F (int) slapMode;
 
 #define SLAP_TRUNCATE_MODE	0x0100
 
+struct slap_replica_info {
+	char   *ri_host;	/* supersedes be_replica */
+	char  **ri_nsuffix;	/* array of suffixes this replica accepts */
+};
+
 /* temporary aliases */
 typedef BackendDB Backend;
 #define nbackends nBackendDB
@@ -903,7 +911,7 @@ struct slap_backend_db {
 	int	be_timelimit;	/* time limit for this backend       	   */
 	AccessControl *be_acl;	/* access control list for this backend	   */
 	slap_access_t	be_dfltaccess;	/* access given if no acl matches	   */
-	char	**be_replica;	/* replicas of this backend (in master)	   */
+	struct slap_replica_info **be_replica;	/* replicas of this backend (in master)	*/
 	char	*be_replogfile;	/* replication log file (in master)	   */
 	char	*be_update_ndn;	/* allowed to make changes (in replicas) */
 	struct berval **be_update_refs;	/* where to refer modifying clients to */
